"use strict";

var assign = require("object-assign"),
    Emitter = require("component-emitter"),
    Immutable = require("immutable");

var core = {
  records: Immutable.List(),
  pointer: 0,
  isTravelling: false,
  initial: Immutable.Map(),
  updater: function updater(state) {
    return state;
  },
  inProduction: false
};
var recorderEmitter = new Emitter();

var callUpdater = function callUpdater(actionType, actionData) {
  var chunks = actionType.split("/");
  var groupName = chunks[0];
  return groupName === "actions-recorder" ? (function () {
    switch (chunks[1]) {
      case "commit":
        return {
          initial: core.records.reduce(function (acc, action) {
            return core.updater(acc, action.get(0), action.get(1));
          }, core.initial),
          records: Immutable.List(),
          pointer: 0,
          isTravelling: false
        };

      case "reset":
        return {
          records: Immutable.List(),
          pointer: 0,
          isTravelling: false
        };

      case "peek":
        return {
          pointer: actionData,
          isTravelling: true
        };

      case "discard":
        return {
          records: core.records.slice(0, core.pointer + 1)
        };

      case "switch":
        return {
          isTravelling: !core.isTravelling,
          pointer: 0
        };

      default:
        console.warn("Unknown actions-recorder action: " + actionType)
        return {};
    }
  })() : {
    records: core.records.push(Immutable.List([actionType, actionData]))
  };
};

var getNewStore = function getNewStore() {
  return core.isTravelling && core.pointer >= 0 ? core.records.slice(0, core.pointer + 1).reduce(function (acc, action) {
    return core.updater(acc, action.get(0), action.get(1));
  }, core.initial) : core.records.reduce(function (acc, action) {
    return core.updater(acc, action.get(0), action.get(1));
  }, core.initial);
};

exports.setup = function (options) {
  return assign(core, options);
};

exports.request = function (fn) {
  return fn(getNewStore(), core);
};

exports.subscribe = function (fn) {
  return recorderEmitter.on("update", fn);
};

exports.unsubscribe = function (fn) {
  return recorderEmitter.off("update", fn);
};

exports.dispatch = function (actionType, actionData) {
  actionData = Immutable.fromJS(actionData);

  if (core.inProduction) {
    core.initial = core.updater(core.initial, actionType, actionData);
    core.records = core.records.push(Immutable.List([actionType, actionData]));
    recorderEmitter.emit("update", core.initial, core);
  } else {
    assign(core, callUpdater(actionType, actionData));
    recorderEmitter.emit("update", getNewStore(), core);
  }

  return;
};